/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.admin.ReportAdminViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.MessageDialog");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.FilteringTable");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.PaginationControl");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");

var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var CoreAdminMessages	    = dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");
var ReportClient			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var currentReportType		= "REPORT_DESIGN"; 
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
var PaginationControl	  	= com.ibm.rational.report.web.core.app.internal.common.PaginationControl;

var startIndex = 0;
var pageSize = 0; 
/**
 * Displays the Manage Reports view.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.admin.ReportAdminViewer",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/admin/templates/ReportAdminViewer.html"),
	
	constructor: function(args) {
		this._controller = args.controller;
	},
	
	postCreate: function() {
		var chk = document.createElement("input");
		chk.type = "checkbox";
		
		var toolbar = new dijit.Toolbar({layoutAlign: "top", style: "background : transparent; border: none"});	
        
		this.downloadButton = new dijit.form.Button({
            title: CommonMessages.DOWNLOAD_REPORTS_TOOLTIP,
            iconClass: "DownloadButtonIcon",
			label: CommonMessages.DOWNLOAD_REPORTS_TOOLTIP,
			showLabel: false,
            onClick: dojo.hitch(this, this._downloadReportItem)
        });
        
		this.deleteButton = new dijit.form.Button({
            title: CommonMessages.DELETE_REPORTS_TOOLTIP,
            iconClass: "DeleteButtonIcon",
			label: CommonMessages.DELETE_REPORTS_TOOLTIP,
			showLabel: false,
            onClick: dojo.hitch(this, this._displayConfirmationDialog)
        });
		
		this.downloadButton.disabled = true;
		this.deleteButton.disabled = true;
		this.downloadButton._setStateClass();
		this.deleteButton._setStateClass();
		
		if (currentReportType == "REPORT_DESIGN") {
			toolbar.addChild(this.downloadButton);
		} 
		toolbar.addChild(this.deleteButton);
		
		this.imageButtons.appendChild(toolbar.domNode);
		
		this._tableColumns = [
			{field: "Select", sortType: "__markup__", label: chk,  width: "12px"},
			{field: "Name", label: CommonMessages.NAME_TABLE_HEADER_LABEL},
			{field: "Description", label: CommonMessages.DESCRIPTION_TABLE_HEADER_LABEL, width: "35%"},
			{field: "Type", label: CommonMessages.TYPE_TABLE_HEADER_LABEL, width: "20%"}
		];
		
		this._initText();
				
		this._msgFormatter = new MessageFormatter(this.messageArea);
		var self = this;
		
		var args = new Object();
		args.viewer = this;
		this.paginationControl = new PaginationControl(args);
		this.paginationButtons.appendChild(this.paginationControl.domNode);
		
		this.reportTypeCombo.value = currentReportType;
		
		dojo.connect(this.reportTypeCombo, "onchange",
			function(e) {
				e.preventDefault();
				currentReportType = self.reportTypeCombo.value;
				self.displayContent();
				self.downloadButton.disabled = true;
				self.deleteButton.disabled = true;
				self.downloadButton._setStateClass();
				self.deleteButton._setStateClass();	
				// Unable to access the toolbar buttons using the keyboard, so deleting all the buttons and
				// creating the toolbar again everytime an option is selected.
				Util.removeChildren(self.imageButtons);
				toolbar = new dijit.Toolbar({layoutAlign: "top", style: "background : transparent; border: none"});
				if (currentReportType == "REPORT_DESIGN") {
					toolbar.addChild(self.downloadButton);
				}
				toolbar.addChild(self.deleteButton);
				self.imageButtons.appendChild(toolbar.domNode);		
			}
		);
		
		this._table = new com.ibm.rational.report.web.core.app.internal.common.FilteringTable(
				{valueField: "rowId", noSelectField: "Select"}, this.reportTable);
			
		for (var i = 0; i < this._tableColumns.length; i++) {
			this._table.columns.push(this._table.createMetaData(this._tableColumns[i]));
		}
			
		dojo.connect(this._table, "onSelect", this, "_onTableSelect");
		dojo.connect(this._table, "onGroupSelect", this, "_onGroupSelect");
		
		this.displayContent();
	},
	
	_initText: function() {
		Util.removeChildren(this.pageTitle);
		this.pageTitle.appendChild(document.createTextNode(CoreAdminMessages.REPORT_ADMIN_VIEWER_PAGE_TITLE));
		
		Util.removeChildren(this.reportTypeLabel);
		this.reportTypeLabel.appendChild(document.createTextNode(CoreAdminMessages.REPORT_ADMIN_VIEWER_REPORT_TYPE_LABEL));
	
		Util.removeChildren(this.optionDesigns);
		this.optionDesigns.appendChild(document.createTextNode(CommonMessages.REPORT_DESIGNS));
	
		Util.removeChildren(this.optionQueries);
		this.optionQueries.appendChild(document.createTextNode(CommonMessages.REPORT_QUERIES));
		
		Util.removeChildren(this.optionSnapshots);
		this.optionSnapshots.appendChild(document.createTextNode(CommonMessages.REPORT_SNAPSHOTS));		
	},
	
	displayContent: function() {
		Util.removeChildren(this.messageArea);
		this._table.store.clearData();
		
		startIndex = this.paginationControl.getStartIndex();
		pageSize = this.paginationControl.getPageSize();
		
		if (currentReportType == "REPORT_DESIGN" || currentReportType == "REPORT_ALL") {
			var srh = new ServiceResponseHandler(this, "_displayReportDesigns", "_getAllReportDesignsError");				
			ReportClient.getReportDesignsPage(srh, {startIndex: startIndex, length: pageSize});
		}
		if (currentReportType == "REPORT_QUERY" || currentReportType == "REPORT_ALL") {
			var srh = new ServiceResponseHandler(this, "_displayReportQueries",	"_getAllReportQueriesError");
			ReportClient.getReportQueriesPage(srh, {startIndex: startIndex, length: pageSize});
		}
		if (currentReportType == "REPORT_SNAPSHOT" || currentReportType == "REPORT_ALL") {
			var srh = new ServiceResponseHandler(this, "_displayReportSnapshots", "_getAllReportSnapshotsError");
			ReportClient.getReportSnapshotsPage(srh, {startIndex: startIndex, length: pageSize});
		}
	},
	
	_disableDownloadButton : function () {
		var noReportDesigns = 0;
		var inputs = this.reportTable.tBodies[0].getElementsByTagName("input");
		dojo.forEach(inputs, function(input) {
			if (input.checked && input.name == "REPORT_DESIGN") {
				noReportDesigns ++;
				if (noReportDesigns > 1)
					return true;    
			}
		});
		return (noReportDesigns != 1);
	},
	
	_setSelectToolTips: function() {
		var inputs = this.reportTable.tBodies[0].getElementsByTagName("input");
		
		dojo.forEach(inputs, function(input) {
			var select = (input.checked)?  CommonMessages.DESELECT_LABEL :
				CommonMessages.SELECT_LABEL;
			var type = "";
			if (input.name == "REPORT_DESIGN")
				type = CommonMessages.REPORT_DESIGN;	
			if (input.name == "REPORT_SNAPSHOT")
				type = 	CommonMessages.REPORT_SNAPSHOT;
			if (input.name == "REPORT_QUERY")
				type = 	CommonMessages.REPORT_QUERY;
			input.title = select + type;		
		});
	},
	
	_onGroupSelect : function() {
		var rows = this._table.getGroupSelectedRows();
		this.downloadButton.disabled = (rows != 1);	
		this.deleteButton.disabled = (rows <= 0);
		this.downloadButton._setStateClass();
		this.deleteButton._setStateClass();	
		this._setSelectToolTips();
	},
	
	_onTableSelect: function() {
		if (this._table.isColumnSelected(0)) {
			return;
		}
			
		var data = this._table.getSelectedData();
		var report = new Object();
		report.type = data.Select.name;
		report.itemId = data.itemId;
		report.name = data.Name;
		report.description = data.Description;
		report.reportDesignType = data.Type;
		report.shared = data.shared;
		report.projectArea = data.projectArea;
		report.teamArea = data.teamArea;
		
		this._controller.editReport(report);
	},
	
	_downloadReportItem: function () {
		var inputs = this.reportTable.tBodies[0].getElementsByTagName("input");
		dojo.forEach(inputs, function(input) {
			if (input.checked && input.name == "REPORT_DESIGN") {    
				var DOWNLOAD_SERVICE_URI = TeamServerClient._getServicePrefix()+"com.ibm.rational.report.common.IReportContentRestService/downloadreport/"+input.id;
				location.href = DOWNLOAD_SERVICE_URI;
				/*
				document.getElementById("reportAdminViewForm").method="GET";
				document.getElementById("reportAdminViewForm").action=DOWNLOAD_SERVICE_URI;
				document.getElementById("reportAdminViewForm").submit();
				*/
			}
		});
	},
	
	_deleteReportItems: function () {
		var inputs = this.reportTable.tBodies[0].getElementsByTagName("input");
		var srh = new ServiceResponseHandler(this, "_refreshPage", "_postDeleteReportItemError");
		
		var reportDeleted = false;
		var reportIds = [];
		dojo.forEach(inputs, function(input) {
			if (input.checked) {
				reportDeleted = true;
				reportIds.push(input.id);
			}
		});
		if (reportDeleted) {
			ReportClient.postDeleteReports(srh,{ids: reportIds});
		}
	},
	
	_refreshPage: function() {
		this.downloadButton.disabled = true;
		this.deleteButton.disabled = true;
		this.downloadButton._setStateClass();
		this.deleteButton._setStateClass();
				
		this._controller.syncReportNodeChanged();
		
		this.displayContent();
	},
	
	_postDeleteReportItemError: function(error) {
		var msg = dojo.string.substitute(
				ServiceabilityMessages.FAILED_TO_DELETE_REPORT, [dojo.trim(this.nameText.value)]);
		this._msgFormatter.displayErrorMessage(msg, error);
	},
	
	/**
	 * Displays all the report designs in a table
	 */
	_displayReportDesigns: function(reportDesigns) {
		this.paginationControl.setPreviousLink(startIndex > 1);
		if (reportDesigns != null) {
			
			// update paginationControl as per the results
			this.paginationControl.setPageInfo(startIndex, startIndex + reportDesigns.length - 1);
			this.paginationControl.setNextLink(reportDesigns.length >= pageSize);
			
			for ( var i = 0; i < reportDesigns.length; i++) {
				var reportDesign = reportDesigns[i];
				var chk = document.createElement("input");
				chk.type = "checkbox";
				chk.id = reportDesign.itemId;
				chk.name = "REPORT_DESIGN";
				chk.title = CommonMessages.SELECT_REPORT_DESIGN_TOOLTIP;
				this._table.store.addData({
					rowId: i,
					itemId: reportDesign.itemId,
					Name: reportDesign.name.replace(" ", "\xa0", "g"), // preserve the spaces
					Name_class: ((reportDesign.shared) ? "dataTable-REPORT_DESIGN_SHARED" : "dataTable-REPORT_DESIGN" ),
					Description: reportDesign.description,
					Type: reportDesign.reportDesignType,
					shared: reportDesign.shared,
					projectArea: reportDesign.projectArea,
					teamArea: reportDesign.teamArea,
					Select: chk
				}, reportDesign.itemId);
			}			
		} else {
			this.paginationControl.setNextLink(false);
		}
	},
	
	_getAllReportDesignsError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_RETRIEVE_REPORT_DESIGNS, error);
	},
	
	/**
	 * Displays all the report queries in a table
	 */
	_displayReportQueries: function(reportQueries) {
		this.paginationControl.setPreviousLink(startIndex > 1);
		if (reportQueries != null) {

			// update paginationControl as per the results
			this.paginationControl.setPageInfo(startIndex, startIndex + reportQueries.length - 1);
			this.paginationControl.setNextLink(reportQueries.length >= pageSize);

			for (var i = 0; i < reportQueries.length; i++) {
				var reportQuery = reportQueries[i];
				var chk = document.createElement("input");
				chk.type = "checkbox";
				chk.id = reportQuery.itemId;
				chk.title = CommonMessages.SELECT_REPORT_QUERY_TOOLTIP;
				chk.name = "REPORT_QUERY";
				this._table.store.addData({
					rowId: i,
					itemId: reportQuery.itemId,
					Name: reportQuery.name.replace(" ", "\xa0", "g"), // preserve the spaces
					Name_class: ((reportQuery.shared) ? "dataTable-REPORT_QUERY_SHARED" : "dataTable-REPORT_QUERY" ),
					Description: reportQuery.description,
					Type: null,
					shared: reportQuery.shared,
					projectArea: reportQuery.projectArea,
					teamArea: reportQuery.teamArea,
					Select: chk
				}, reportQuery.itemId);
			}
		} else {
			this.paginationControl.setNextLink(false);
		}
	},
	
	_getAllReportQueriesError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_RETRIEVE_REPORT_QUERIES, error);
	},
	
	/**
	 * Displays all the report snapshots in a table
	 */
	_displayReportSnapshots: function(reportSnapshots) {
		this.paginationControl.setPreviousLink(startIndex > 1);
		if (reportSnapshots != null) {
			// update paginationControl as per the results			
			this.paginationControl.setPageInfo(startIndex, startIndex + reportSnapshots.length - 1);
			this.paginationControl.setNextLink(reportSnapshots.length >= pageSize);

			for (var i = 0; i < reportSnapshots.length; i++) {
				var reportSnapshot = reportSnapshots[i];
				var chk = document.createElement("input");
				chk.type = "checkbox";
				chk.id = reportSnapshot.itemId;
				chk.title = CommonMessages.SELECT_REPORT_SNAPSHOT_TOOLTIP;
				chk.name = "REPORT_SNAPSHOT";
				this._table.store.addData({
					rowId: i,
					itemId: reportSnapshot.itemId,
					Name: reportSnapshot.name.replace(" ", "\xa0", "g"), // preserve the spaces,
					Name_class: ((reportSnapshot.shared) ? "dataTable-REPORT_SNAPSHOT_SHARED" : "dataTable-REPORT_SNAPSHOT"),
					Description: reportSnapshot.description,
					Type: null,
					shared: reportSnapshot.shared,
					projectArea: reportSnapshot.projectArea,
					teamArea: reportSnapshot.teamArea,
					Select: chk
				}, reportSnapshot.itemId);
			}
		} else {
			this.paginationControl.setNextLink(false);
		}
	},
	
	_getAllReportSnapshotsError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_RETRIEVE_REPORT_SNAPSHOTS, error);
	},
	
	_displayConfirmationDialog: function() {
		var self = this;
		var prompt;
		if (currentReportType == "REPORT_ALL"){
			prompt = CommonMessages.CONFIRMATION_DELETE_REPORTS;
		}
		else if (currentReportType == "REPORT_DESIGN"){
			prompt = CommonMessages.CONFIRMATION_DELETE_REPORT_DESIGNS;
		}
		else if (currentReportType == "REPORT_QUERY"){
			prompt = CommonMessages.CONFIRMATION_DELETE_REPORT_QUERIES;
		}
		else if (currentReportType == "REPORT_SNAPSHOT"){
			prompt = CommonMessages.CONFIRMATION_DELETE_REPORT_SNAPSHOTS;
		}
		var confirmationDialog = new com.ibm.rational.report.web.core.app.internal.common.MessageDialog(
			{title: CommonMessages.CONFIRMATION, prompt: prompt, promptType: "Warning", buttonType: "YesNo"});
		dojo.connect(confirmationDialog.leftButton, "onclick", confirmationDialog.dialog, function (e) {
				dojo.stopEvent(e);
				self._deleteReportItems();
				confirmationDialog.close();
		});	
		dojo.connect(confirmationDialog.rightButton, "onclick", confirmationDialog.dialog, function (e) {
				dojo.stopEvent(e);
				confirmationDialog.close();
		});
	}
	
});

})();